@@ -1,18 +1,40 @@
 import logging
 from collections import OrderedDict
-from typing import Any, Iterable, List, Optional, Union
+from typing import Any, Iterable, Dict, List, Optional, Union, NamedTuple
 
 import numpy
 
 from cosapp.core.numerics import sobol_seq
-from cosapp.ports.port import BasePort, PortType
+from cosapp.ports.port import BasePort
 from cosapp.drivers.abstractsetofcases import AbstractSetOfCases
 from cosapp.drivers.abstractsolver import AbstractSolver
+from cosapp.utils.distributions import Distribution
 from cosapp.utils.helpers import check_arg
 
 logger = logging.getLogger(__name__)
 
 
+class RandomVariable(NamedTuple):
+    variable: "VariableReference"
+    distribution: Distribution
+    connector: Optional["SystemConnector"] = None
+
+    def add_noise(self, quantile=None) -> float:
+        delta = self.draw(quantile)
+        self.set_perturbation(delta)
+        return delta
+
+    def draw(self, quantile=None) -> float:
+        return self.distribution.draw(quantile)
+
+    def set_perturbation(self, value) -> None:
+        connector = self.connector
+        if connector is None:
+            self.variable.value += value
+        else:
+            connector.set_perturbation(self.variable.key, value)
+
+
 # TODO linearization does not support multipoint cases
 # TODO Does not work for vector variables (at least partially connected one)
 # TODO Does it work if a subsystem mutates to higher/lower fidelity
@@ -49,7 +71,7 @@ class MonteCarlo(AbstractSetOfCases):
         self.linear = False  # type: bool
             # desc="True for linearisation of system before Montecarlo calculation. Default False."
 
-        self.random_variables = OrderedDict()  # type: Dict[str, Tuple[VariableReference, Optional[Connector], Distribution]]
+        self.random_variables: Dict[str, RandomVariable] = OrderedDict()
             # desc="Random variables in the system."
         self.responses = list()  # type: List[str]
             # We need a list as set is not ordered
@@ -83,36 +105,35 @@ class MonteCarlo(AbstractSetOfCases):
             List of variables to be perturbated
         """
         # TODO it should be possible to set the distribution directly
+        name2variable = self.owner.name2variable
 
         def add_unique_input_var(name: str):
             self.check_owner_attr(name)
-            ref = self.owner.name2variable[name]
+            ref = name2variable[name]
             port = ref.mapping
 
             if not isinstance(port, BasePort):
                 raise TypeError(f"{name!r} is not a variable.")
 
-            if port.direction != PortType.IN:
-                raise TypeError(
-                    f"{ref.key!r} is not an input variable of {port.contextual_name!r}"
-                )
+            if not port.is_input:
+                raise TypeError(f"{name!r} is not an input variable.")
 
             distribution = port.get_details(ref.key).distribution
             if distribution is None:
                 raise ValueError(
-                    f"No distribution specified for '{port.contextual_name}.{ref.key}'"
+                    f"No distribution specified for {name!r}"
                 )
 
             # Test if the variable is connected
             connection = None
             if port.owner.parent is not None:
-                connectors = port.owner.parent.connectors.values()
+                connectors = port.owner.parent.all_connectors()
                 for connector in filter(lambda c: c.sink is port, connectors):
-                    if ref.key in connector.mapping:
+                    if ref.key in connector.sink_variables():
                         connection = connector
                         break
 
-            self.random_variables[name] = (ref, connection, distribution)
+            self.random_variables[name] = RandomVariable(ref, distribution, connection)
 
         check_arg(names, 'names', (str, set, list))
 
@@ -120,7 +141,7 @@ class MonteCarlo(AbstractSetOfCases):
             add_unique_input_var(names)
         else:
             for name in names:
-                check_arg(name, name + " in 'names'", str)
+                check_arg(name, f"{name} in 'names'", str)
                 add_unique_input_var(name)
 
     def add_response(self, name: Union[str, Iterable[str]]) -> None:
@@ -142,7 +163,7 @@ class MonteCarlo(AbstractSetOfCases):
             add_unique_response_var(name)
         else:
             for n in name:
-                check_arg(n, n + " in 'name'", str)
+                check_arg(n, f"{n} in 'name'", str)
                 add_unique_response_var(n)
 
     def _build_cases(self) -> None:
@@ -212,13 +233,9 @@ class MonteCarlo(AbstractSetOfCases):
 
         # Set perturbation
         self.perturbations = numpy.zeros(len(self.random_variables))
-        for i, params in enumerate(self.random_variables.values()):
-            ref, connector, distribution = params
-            self.perturbations[i] = distribution.draw(case[i])
-            if connector is None:
-                ref.value += self.perturbations[i]
-            else:
-                connector.set_perturbation(ref.key, self.perturbations[i])
+        for i, variable in enumerate(self.random_variables.values()):
+            perturbation = variable.add_noise(case[i])
+            self.perturbations[i] = perturbation
 
         if len(self.reference_case_solution) > 0:
             self.solver.load_solution(self.reference_case_solution)
@@ -262,9 +279,8 @@ class MonteCarlo(AbstractSetOfCases):
         super()._postcase(case_idx, case)
 
         # Remove the perturbation
-        for i, params in enumerate(self.random_variables.values()):
-            ref, connector = params[:2]
-            if connector is None:
-                ref.value -= self.perturbations[i]
+        for variable, delta in zip(self.random_variables.values(), self.perturbations):
+            if variable.connector is None:
+                variable.set_perturbation(-delta)
             else:
-                connector.set_perturbation(ref.key, -self.perturbations[i])
+                variable.connector.clear_noise()
