@@ -12,7 +12,6 @@ from .multiply import (
     NonLinear1,
     NonLinear3,
     IterativeNonLinear,
-    Multiply2Derivative,
     IterativeNonLinearDerivative,
 )
 from .multiply import Splitter as SplitterMath
@@ -52,3 +51,12 @@ from .vectors import (
     AllTypesSystem,
     BooleanSystem,
 )
+from .elec import (
+    Voltage,
+    Current,
+    Dipole,
+    Resistor,
+    Node,
+    Source,
+    Ground,
+)
