@@ -4,7 +4,7 @@ from cosapp.tools.problem_viewer.problem_viewer import view_model
 from cosapp.tools.views.d3js import to_d3
 from cosapp.tools.views.markdown import port_to_md, system_to_md
 from cosapp.tools.views.prettyprint import list_inputs, list_outputs
-from cosapp.tools.views.visjs import to_visjs
+from cosapp.tools.views.visjs import VisJsRenderer, to_visjs
 
 __all__ = [
     "display_doc",
@@ -16,4 +16,5 @@ __all__ = [
     "to_d3",
     "to_fmu",
     "to_visjs",
+    "VisJsRenderer",
 ]
